<?php

namespace Drupal\frontend_editing\Hook;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Render\ElementInfoManager;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\frontend_editing\ToolbarItem;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Hook implementations for frontend_editing.
 */
class FrontendEditingHooks {

  /**
   * Constructs FrontendEditing hooks.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory object.
   * @param \Drupal\Core\Routing\AdminContext $adminContext
   *   The admin context.
   * @param \Drupal\Core\Render\ElementInfoManager $elementInfoManager
   *   The element info manager.
   */
  public function __construct(
    #[Autowire(service: 'current_user')]
    protected AccountProxyInterface $currentUser,
    #[Autowire(service: 'config.factory')]
    protected ConfigFactoryInterface $configFactory,
    #[Autowire(service: 'router.admin_context')]
    protected AdminContext $adminContext,
    #[Autowire(service: 'plugin.manager.element_info')]
    protected ElementInfoManager $elementInfoManager,
  ) {}

  /**
   * Implements hook_toolbar().
   */
  #[Hook('toolbar')]
  public function toolbar() {
    $items = [];
    $cache_metadata = new CacheableMetadata();
    $cache_metadata->addCacheContexts(['user.permissions']);
    if (!$this->currentUser->hasPermission('access frontend editing')) {
      $cache_metadata->applyTo($items);
      return $items;
    }

    $config = $this->configFactory->get('frontend_editing.settings');
    $cache_metadata->addCacheableDependency($config);
    $cache_metadata->addCacheContexts(['route']);
    $ui_toggle_enabled = $config->get('ui_toggle');
    if ($ui_toggle_enabled || $this->adminContext->isAdminRoute()) {
      $cache_metadata->applyTo($items);
      return $items;
    }

    $items['toggle'] = [
      '#type' => 'toolbar_item',
      'tab' => [
        '#lazy_builder' => [
          'frontend_editing.toolbar_item:renderToggle',
          [],
        ],
        '#create_placeholder' => TRUE,
        '#cache' => [
          'tags' => [
            'frontend_editing:toggle',
          ],
        ],
      ],
      '#wrapper_attributes' => [
        'class' => [
          'frontend-editing-toolbar-toggle',
        ],
      ],
      '#weight' => -11,
    ];
    // \Drupal\toolbar\Element\ToolbarItem::preRenderToolbarItem adds an
    // #attributes property to each toolbar item's tab child automatically.
    // Lazy builders don't support an #attributes property so we need to
    // add another render callback to remove the #attributes property. We start
    // by adding the defaults, and then we append our own pre render callback.
    $items['toggle'] += $this->elementInfoManager->getInfo('toolbar_item');
    $items['toggle']['#pre_render'][] = [ToolbarItem::class, 'removeTabAttributes'];
    $cache_metadata->applyTo($items);
    return $items;
  }

}
